@@ -23,6 +23,7 @@
 
 //--own headers
 #include "gpe-sketchbook.h"
+#include "preferences.h"
 #include "sketchpad.h"
 #include "sketchpad-gui.h"
 #include "sketchpad-cb.h" //on_window_sketchpad_destroy()
@@ -46,8 +47,10 @@ static struct gpe_icon my_icons[] = {
   //GPE stock icons
   { "new",        "new"   },
   { "open",       "edit"  },
+  { "import",     "open"  },
   { "delete",     "delete"},
   { "save",       "save"  },
+  { "prefs",      "preferences"  },
   { "properties", "properties"  },
   { "left",       "left"  },
   { "right",      "right" },
@@ -88,6 +91,8 @@ void app_init(int argc, char ** argv){
   sketchbook.save_dir = g_strdup_printf("%s/%s/", g_get_home_dir(), SKETCHPAD_HOME_DIR);
   selector_init();
   sketchpad_init();
+  prefs_reset_defaults();
+  prefs_fetch_settings();
 }
 
 void app_quit(){
@@ -100,12 +105,15 @@ void on_notebook_switch_page(GtkNotebook * notebook,
                              gpointer unused){
   //--update window title
   switch (page_number){
-  case PAGE_SKETCHPAD:
-    sketchpad_reset_title();
-    break;
-  case PAGE_SELECTOR:
-  default:
-    gtk_window_set_title (GTK_WINDOW (sketchbook.window), _("Sketchbook"));
+    case PAGE_SKETCHPAD:
+      sketchpad_reset_title();
+      break;
+    case PAGE_PREFERENCES:
+      gtk_window_set_title (GTK_WINDOW (sketchbook.window), _("Sketchbook: preferences"));
+      break;
+    case PAGE_SELECTOR:
+    default:
+      gtk_window_set_title (GTK_WINDOW (sketchbook.window), _("Sketchbook"));
   }
 }
 
@@ -115,6 +123,7 @@ void gui_init(){
 
   GtkWidget * selector_ui;
   GtkWidget * sketchpad_ui;
+  GtkWidget * prefs_ui;
 
   //--toplevel window
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
@@ -137,6 +146,9 @@ void gui_init(){
   sketchpad_ui = sketchpad_gui(window);
   window_sketchpad_init(window);
 
+  //--preferences
+  prefs_ui = preferences_gui(window);
+
   //--packing
   notebook = (GtkNotebook *) gtk_notebook_new();
   sketchbook.notebook = notebook;
@@ -146,6 +158,7 @@ void gui_init(){
 
   gtk_notebook_insert_page(notebook, selector_ui,  NULL, PAGE_SELECTOR);
   gtk_notebook_insert_page(notebook, sketchpad_ui, NULL, PAGE_SKETCHPAD);
+  gtk_notebook_insert_page(notebook, prefs_ui,     NULL, PAGE_PREFERENCES);
 
 #ifdef DEBUG
   {
